@@ -14,6 +14,7 @@
    limitations under the License.
 """
 import numpy as np
+import scipy.sparse as sp
 from collections import defaultdict
 from sageopt.coniclifts.constraints.elementwise import ElementwiseConstraint
 from sageopt.coniclifts.constraints.set_membership.psd_cone import PSD
@@ -266,10 +267,15 @@ class ScalarExpression(object):
             return f
         elif isinstance(other, np.ndarray):
             return self.as_expr() * other
-        elif isinstance(other, ScalarExpression) and other.is_constant():
+        if not isinstance(other, ScalarExpression):
+            otype = str(type(other))
+            raise RuntimeError('Cannot multiply ScalarExpression with object of type %s', otype)
+        elif other.is_constant():
             return other.offset * self
+        elif self.is_constant():
+            return other * self.offset
         else:
-            return other.__rmul__(self)
+            raise RuntimeError('Cannot multiply two non-constant ScalarExpression objects.')
 
     def __truediv__(self, other):
         if not isinstance(other, __REAL_TYPES__):  # pragma: no cover
@@ -301,6 +307,7 @@ class ScalarExpression(object):
                 del self.atoms_to_coeffs[a]
 
     def is_constant(self):
+        self.remove_zeros()
         return len(self.atoms_to_coeffs) == 0
 
     def is_convex(self):
@@ -342,6 +349,7 @@ class ScalarExpression(object):
         return ElementwiseConstraint(self, other, "==")
 
     def scalar_variables(self):
+        self.remove_zeros()
         svs = []
         for a in self.atoms_to_coeffs:
             if isinstance(a, ScalarVariable):
@@ -404,6 +412,8 @@ class Expression(np.ndarray):
 
     """
 
+    __array_priority__ = 100
+
     def __new__(cls, obj):
         attempt = np.array(obj, dtype=object, copy=False, subok=True)
         for tup in array_index_iterator(attempt.shape):
@@ -431,6 +441,8 @@ class Expression(np.ndarray):
             msg = '\n \t Matmul implementation uses "dot", '
             msg += 'which behaves differently for higher dimension arrays.\n'
             raise RuntimeError(msg)
+        if isinstance(other, sp.spmatrix):
+            other = other.toarray()
         (A, x, B) = self.factor()
         if isinstance(other, Expression):
             if not other.is_constant():  # pragma: no cover
@@ -848,6 +860,7 @@ class Variable(Expression):
         return True
 
     def scalar_variables(self):
+        #TODO: make this faster for "proper" Variable objects.
         return [list(se.atoms_to_coeffs)[0] for se in self.flat]
 
     def leading_scalar_variable_id(self):
@@ -907,6 +920,8 @@ class Variable(Expression):
 
     @value.setter
     def value(self, value):
+        if isinstance(value, __REAL_TYPES__):
+            value = np.array(value)
         if value.shape != self.shape:  # pragma: no cover
             raise RuntimeError('Dimension mismatch.')
         for tup in array_index_iterator(self.shape):
