@@ -42,7 +42,7 @@ SETTINGS = {
     'presolve_trivial_age_cones': False,
     'reduction_solver': 'ECOS',
     'sum_age_force_equality': False,
-    'compact_dual': False,
+    'compact_dual': True,
     'kernel_basis': False,
 }
 
@@ -92,6 +92,14 @@ class PrimalSageCone(SetMembership):
     Other Parameters
     ----------------
 
+    settings : Dict[str, Union[str, bool]]
+
+        A dict for overriding default settings which control how this PrimalSageCone is compiled
+        into the coniclifts standard form. Recognized boolean flags are "heuristic_reduction",
+        "presolve_trivial_age_cones", "sum_age_force_equality", and "kernel_basis". The
+        "kernel_basis" flag is only applicable when ``X=None``. The string flag "reduction_solver"
+        must be either "MOSEK" or "ECOS".
+
     covers : Dict[int, ndarray]
 
         ``covers[i]`` indicates which indices ``j`` have ``alpha[j,:]`` participate in
@@ -155,10 +163,17 @@ class PrimalSageCone(SetMembership):
         Uniquely identifies this Constraint in the model where it appears. Serves as a suffix
         for the name of any auxiliary Variable created when compiling to the coniclifts standard.
 
+    settings : Dict[str, Union[str, bool]]
+
+        Specifies compilation options. By default this dict caches global compilation options
+        when this object is constructed. The global compilation options are overridden
+        if a user supplies ``settings`` as an argument when constructing this PrimalSageCone.
+
     ech : ExpCoverHelper
 
-        A simple wrapper around the constructor argument ``covers``. Validates ``covers``
-        when provided, and constructs ``covers`` when a user does not provide it.
+        A data structure which summarizes the results from a presolve phase. The most important
+        properties of ``ech`` can be specified by providing a dict-valued keyword argument
+        "``covers``" to the PrimalSageCone constructor.
 
     Notes
     -----
@@ -328,7 +343,7 @@ class PrimalSageCone(SetMembership):
                 if not self.settings['kernel_basis']:
                     # linear equality constraints
                     mat = (self.alpha[idx_set, :] - self.alpha[i, :]).T
-                    av, ar, ac = comp_aff.mat_times_vecvar(mat, self._nus[i])
+                    av, ar, ac, _ = comp_aff.matvec(mat, self._nus[i])
                     num_rows = mat.shape[0]
                     curr_b = np.zeros(num_rows, )
                     curr_k = [Cone('0', num_rows)]
@@ -362,7 +377,7 @@ class PrimalSageCone(SetMembership):
                 mat2 = -self.X.A.T
                 var1 = self._nus[i]
                 var2 = self._eta_vars[i]
-                av, ar, ac = comp_aff.mat_times_vecvar_plus_mat_times_vecvar(mat1, var1, mat2, var2)
+                av, ar, ac, _ = comp_aff.matvec_plus_matvec(mat1, var1, mat2, var2)
                 num_rows = mat1.shape[0]
                 curr_b = np.zeros(num_rows, )
                 curr_k = [Cone('0', num_rows)]
@@ -534,11 +549,6 @@ class DualSageCone(SetMembership):
     Other Parameters
     ----------------
 
-    covers : Dict[int, ndarray]
-
-        ``covers[i]`` indicates which indices ``j`` have ``alpha[j,:]`` participate in
-        the i-th AGE cone. Automatically constructed in a presolve phase, if not provided.
-
     c : Expression or None
 
         When provided, this DualSageCone instance will compile to a constraint to ensure that ``v``
@@ -546,6 +556,18 @@ class DualSageCone(SetMembership):
         If we have have information about the sign of a component of  ``c``, then it is possible to
         reduce the number of coniclifts primitives needed to represent this constraint.
 
+    settings : Dict[str, Union[str, bool]]
+
+        A dict for overriding default settings which control how this DualSageCone is compiled
+        into the coniclifts standard form. Recognized boolean flags are "heuristic_reduction",
+        "presolve_trivial_age_cones", and "compact_dual". The string flag "reduction_solver"
+        must be either "MOSEK" or "ECOS".
+
+    covers : Dict[int, ndarray]
+
+        ``covers[i]`` indicates which indices ``j`` have ``alpha[j,:]`` participate in
+        the i-th AGE cone. Automatically constructed in a presolve phase, if not provided.
+        See also ``DualSageCone.ech``.
 
     Attributes
     ----------
@@ -569,15 +591,17 @@ class DualSageCone(SetMembership):
         solution recovery algorithm takes these variables, and considers points ``x`` of
         the form ``x = mu_vars[i].value / self.v[i].value``.
 
-    name : str
+    settings : Dict[str, Union[str, bool]]
 
-        Uniquely identifies this Constraint in the model where it appears. Serves as a suffix
-        for the name of any auxiliary Variable created when compiling to the coniclifts standard.
+        Specifies compilation options. By default this dict caches global compilation options
+        when this object is constructed. The global compilation options are overridden
+        if a user supplies ``settings`` as an argument when constructing this DualSageCone.
 
     ech : ExpCoverHelper
 
-        A simple wrapper around the constructor argument ``covers``. Validates ``covers``
-        when provided, and constructs ``covers`` when a user does not provide it.
+        A data structure which summarizes the results from a presolve phase. The most important
+        properties of ``ech`` can be specified by providing a dict-valued keyword argument
+        "``covers``" to the DualSageCone constructor.
     """
 
     def __init__(self, v, alpha, X, name, **kwargs):
@@ -651,7 +675,7 @@ class DualSageCone(SetMembership):
                     cd = elementwise_relent(expr, self.v[idx_set], epi)
                     cone_data.append(cd)
                     # Linear inequalities
-                    av, ar, ac = comp_aff.mat_times_vecvar_minus_vecvar(mat, vecvar, epi)
+                    av, ar, ac, _ = comp_aff.matvec_minus_vec(mat, vecvar, epi)
                     num_rows = mat.shape[0]
                     curr_b = np.zeros(num_rows)
                     curr_k = [Cone('+', num_rows)]
@@ -661,7 +685,7 @@ class DualSageCone(SetMembership):
                     A, b, K = self.X.A, self.X.b, self.X.K
                     vecvar = self._lifted_mu_vars[i]
                     singlevar = self.v[i]
-                    av, ar, ac = comp_aff.mat_times_vecvar_plus_vec_times_singlevar(A, vecvar, b, singlevar)
+                    av, ar, ac, _ = comp_aff.matvec_plus_vec_times_scalar(A, vecvar, b, singlevar)
                     curr_b = np.zeros(b.size, )
                     cone_data.append((av, ar, ac, curr_b, K))
             return cone_data
