@@ -218,6 +218,10 @@ class Problem(object):
             cache_apply_data : bool. If ``True``, then take the put``(data, inv_data)``
             returned by the coniclifts interface to the given solver and record it in the
             dictionary ``self.solver_apply_data``. This data is rarely relevant for users.
+
+            dualize : bool. Overrides coniclifts automatic dualization procedure when using MOSEK
+            Setting ``dualize=True`` forces dualization, and setting ``dualize=False`` prohibits
+            dualization. Not applicable when solving mixed-integer problems.
         """
         if solver is None:
             for svr in Problem._SOLVER_ORDER_:
@@ -235,7 +239,9 @@ class Problem(object):
 
         # Finish solver-specific compilation
         t0 = time.time()
-        data, inv_data = solver_object.apply(self.c, self.A, self.b, self.K)
+        if self._integer_indices is not None:
+            options['integers'] = True
+        data, inv_data = solver_object.apply(self.c, self.A, self.b, self.K, options)
         self.timings[solver]['apply'] = time.time() - t0
         if self._integer_indices is not None:
             data['integer_indices'] = self._integer_indices.tolist()
